/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * MooDS: a communication protocol used in GASP.
 * GASP Demonstrator: a gasp demonstration game.
 *
 * Copyright (C) 2004 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Franck Caron, Romain Pellerin
 */
/*
 * Created on 2 december 2005
 *
 * @author CARON Franck - CNAM CEDRIC
 * @author PELLERIN Romain - CNAM CEDRIC
 * @author DELPIANO Fabien - Filao
 * 
 * 
 * Research project: CNAM-CEDRIC - INT - Filao
 */

package app2.client.game;

import app2.NewPlayer;
import app2.UpdatePlayer;
import app2.Delay;
import app2.client.App;
import app2.client.IHMGasp;
import app2.client.NetworkCom;


public class SimpleGameCanvas extends GameCanvas {

	private HeroeActor heroe;
	private static UpdatePlayer updp = new UpdatePlayer();	
	/**
	 * @param midlet
	 */
	public SimpleGameCanvas(App midlet, IHMGasp ihmGasp, int _time) {
		super(midlet, ihmGasp,_time);
	}


	/*********************************/
	/*** SimpleGame initialization ***/
	/*********************************/

	protected void initializeNewGame() {
		
		if (heroe == null) {
			heroe = new HeroeActor(this);
			heroe.username = ihmGasp.pseudoname;
		} else {
			heroe.initialize();
		}
		addActor(heroe);
		sendNewPlayer();
		canUpdate = true;
	}

	public Object cyclicUpdateMessagesToSend() {
		if (canUpdate && heroe.getPathPtr()>0){
			updp.setId(ihmGasp.aSID);
			updp.setX(heroe.getX());
			updp.setY(heroe.getY());
			updp.setSpeedX(heroe.getSpeedX());
			updp.setSpeedY(heroe.getSpeedY());

			int hptr=heroe.getPathPtr(),nWpts=Math.min(hptr,UpdatePlayer.MAX_UPDATE_WAYPOINTS);
			for (int i=0; i<nWpts; i++)
			  updp.setCoord(i,heroe.getXCoord(i*hptr/nWpts),heroe.getYCoord(i*hptr/nWpts));
			updp.setPathPtr(nWpts);
			heroe.resetPath();
			canUpdate = false;
			return updp;
		}
		else return null;
	}
	
	/**
	 * This method sends a newPlayer message. It's called by  
	 * initializeNewGame
	 */
	private void sendNewPlayer() {
		NewPlayer np = NetworkCom.newNewPlayerMessage();
		np.setId(ihmGasp.aSID);
		np.setX(heroe.getX());
		np.setY(heroe.getY());
		np.setUsername(heroe.username);
		np.draw();
		ihmGasp.sendMessage(np);
	}
	
	public void initialize() {
		for (int i = 1 ; i <= GameCanvas.NB_OF_PNGS ; i++) {
		try{
		ResourcesManager.preloadResource(i , ResourcesManager.PNG);
			}
		catch (Exception e){
			System.out.println(i);
			}
		}
	ResourcesManager.loadEverything();
	}
	
	protected void manageCamera() {
		//if (heroe != null) centerOnActorX(heroe);
	}

	/***************************/
	/*** Messages management ***/
	/***************************/
	
	/**
	 * This method treats each type of message:
	 * UpdatePlayer, NewPlayer, PingPlayer 
	 */
	public void treatReceivedMessage(Object o) {
		if (o instanceof UpdatePlayer){
		try{
			UpdatePlayer updp = (UpdatePlayer) o;
			MultiplayerActor ma = getMultiplayerActor(updp.getId());
			if (ma!=null)
			{
				ma.setPathPtr(updp.getPathPtr());
				ma.setPathXY(updp.getPathX(),updp.getPathY());

				ma.setWantedPosition(updp.getX(), updp.getY());
				ma.setWantedPositionSpeeds(updp.getSpeedX(), updp.getSpeedY());
				ma.isPositionReached = false;
			}
			System.out.println("Updating player path with "+updp.getPathPtr()+" waypoints.");
			for (int i=0; i<updp.getPathPtr(); i++)
				System.out.println("Point "+i+": "+updp.getXCoord(i)+";"+updp.getYCoord(i));
		}catch(NullPointerException ne){ }
		}
		else if (o instanceof Delay)
		{
			Delay delay=(Delay)o;
			System.out.println("!!! Delay between me and player "+delay.getId()+" found to be "+(delay.getPing()+NetworkCom.averagePing));
		}
		else if (o instanceof NewPlayer) {
		try{
		NewPlayer np = (NewPlayer) o;
		np.draw();
		MultiplayerActor ma = new MultiplayerActor(this,np.getId());
		nbMultiplayerActors++;
		ma.username = np.getUsername();
		System.out.println("Player Name:"+np.getUsername());
		ma.initialize();
		addActor(ma);	
		}catch(NullPointerException ne){ System.out.println("here");}
	}
	}
	}
